'''
Created on Feb 3, 2015

@author: Rui, Gong

'''
from django.http import *
import json

import logging
logger = logging.getLogger("api")

def Http200(request, msg=u"Success",data=""):
    error = {
             u"code": u"200",
             u"message": msg,
             u"data": data
    }
    result = json.dumps(error)
#     logmsg = "user:%s failed with error:%s "%(request.user, msg)
#     logger.info(logmsg)
    return HttpResponse(result, content_type="application/json")

def Http400(request, msg=u"Bad Request",data=""):
    error = {
             u"code": u"400",
             u"message": msg,
             u"data": data
    }
    result = json.dumps(error)
    return HttpResponseBadRequest(result, content_type="application/json")

def Http401(request, msg=u"Unauthorized",data=""):
    error = {
             u"code": u"401",
             u"message": msg,
             u"data": data
    }
    result = json.dumps(error)
    return HttpResponseBadRequest(result, content_type="application/json")

def Http403(request, msg=u"Forbidden",data=""):
    error = {
             u"code": u"403",
             u"message": msg,
             u"data": data
    }
    result = json.dumps(error)
    return HttpResponseForbidden(result, content_type="application/json")

def Http404(request, msg=u"File not found",data=""):
    error = {
             u"code": u"404",
             u"message": msg,
             u"data": data
    }
    result = json.dumps(error)
    return HttpResponseNotFound(result, content_type="application/json")

def Http500(request, msg=u"Internal Server Error",data=""):
    error = {
             u"code": u"500",
             u"message": msg,
             u"data": data
    }
    result = json.dumps(error)
    return HttpResponseServerError(result, content_type="application/json")




#!/usr/bin/env python
#-*- encoding:utf-8 -*-

""" 对公众平台发送给公众账号的消息加解密示例代码.
@copyright: Copyright (c) 1998-2014 Tencent Inc.

"""
# ------------------------------------------------------------------------

import base64
import string
import random
import hashlib
import time
import struct
from Crypto.Cipher import AES
import xml.etree.cElementTree as ET  
import sys                                                                                                                                                                             
import socket
reload(sys)
import ierror
import logging
sys.setdefaultencoding('utf-8') 

"""
关于Crypto.Cipher模块，ImportError: No module named 'Crypto'解决方案
请到官方网站 https://www.dlitz.net/software/pycrypto/ 下载pycrypto。
下载后，按照README中的“Installation”小节的提示进行pycrypto安装。
"""
class FormatException(Exception):
    pass

def throw_exception(message, exception_class=FormatException):
    """my define raise exception function"""
    raise exception_class(message)

class SHA1:
    """计算公众平台的消息签名接口"""   
    
    def getSHA1(self, token, timestamp, nonce, encrypt):
        """用SHA1算法生成安全签名
        @param token:  票据
        @param timestamp: 时间戳
        @param encrypt: 密文
        @param nonce: 随机字符串
        @return: 安全签名
        """
        try:
            sortlist = [token, timestamp, nonce]
            sortlist.sort()
            sha = hashlib.sha1()
            sha.update("".join(sortlist))
            return  ierror.WXBizMsgCrypt_OK, sha.hexdigest()
        except Exception,e:
            print e
            return  ierror.WXBizMsgCrypt_ComputeSignature_Error, None
  

class XMLParse:
    """提供提取消息格式中的密文及生成回复消息格式的接口"""   
     
    # xml消息模板   
    AES_TEXT_RESPONSE_TEMPLATE = """<xml>
<Encrypt><![CDATA[%(msg_encrypt)s]]></Encrypt>
<MsgSignature><![CDATA[%(msg_signaturet)s]]></MsgSignature>
<TimeStamp>%(timestamp)s</TimeStamp>
<Nonce><![CDATA[%(nonce)s]]></Nonce>
</xml>"""

    def extract(self, xmltext):
        """提取出xml数据包中的加密消息 
        @param xmltext: 待提取的xml字符串
        @return: 提取出的加密消息字符串
        """
        try:
            xml_tree = ET.fromstring(xmltext)
            encrypt  = xml_tree.find("Encrypt")
            touser_name    = xml_tree.find("ToUserName")
            return  ierror.WXBizMsgCrypt_OK, encrypt.text, touser_name.text
        except Exception,e:
            print e
            return  ierror.WXBizMsgCrypt_ParseXml_Error,None,None
    
    def generate(self, encrypt, signature, timestamp, nonce):
        """生成xml消息
        @param encrypt: 加密后的消息密文
        @param signature: 安全签名
        @param timestamp: 时间戳
        @param nonce: 随机字符串
        @return: 生成的xml字符串
        """
        resp_dict = {
                    'msg_encrypt' : encrypt,
                    'msg_signaturet': signature,
                    'timestamp'    : timestamp,
                    'nonce'        : nonce,
                     }
        resp_xml = self.AES_TEXT_RESPONSE_TEMPLATE % resp_dict
        return resp_xml   
    
 
class PKCS7Encoder():
    """提供基于PKCS7算法的加解密接口"""  
    
    block_size = 32
    def encode(self, text):
        """ 对需要加密的明文进行填充补位
        @param text: 需要进行填充补位操作的明文
        @return: 补齐明文字符串
        """
        text_length = len(text)
        # 计算需要填充的位数
        amount_to_pad = self.block_size - (text_length % self.block_size)
        if amount_to_pad == 0:
            amount_to_pad = self.block_size
        # 获得补位所用的字符
        pad = chr(amount_to_pad)
        return text + pad * amount_to_pad
    
    def decode(self, decrypted):
        """删除解密后明文的补位字符
        @param decrypted: 解密后的明文
        @return: 删除补位字符后的明文
        """
        pad = ord(decrypted[-1])
        if pad<1 or pad >32:
            pad = 0
        return decrypted[:-pad]
    
    
class Prpcrypt(object):
    """提供接收和推送给公众平台消息的加解密接口"""
    
    def __init__(self,key):

        #self.key = base64.b64decode(key+"=")
        self.key = key
        # 设置加解密模式为AES的CBC模式   
        self.mode = AES.MODE_CBC
    
            
    def encrypt(self,text,corpid):
        """对明文进行加密
        @param text: 需要加密的明文
        @return: 加密得到的字符串
        """      
        # 16位随机字符串添加到明文开头
        text = self.get_random_str() + struct.pack("I",socket.htonl(len(text))) + text + corpid
        # 使用自定义的填充方式对明文进行补位填充
        pkcs7 = PKCS7Encoder()
        text = pkcs7.encode(text)
        # 加密    
        cryptor = AES.new(self.key,self.mode,self.key[:16])
        try:
            ciphertext = cryptor.encrypt(text)
            # 使用BASE64对加密后的字符串进行编码
            return ierror.WXBizMsgCrypt_OK, base64.b64encode(ciphertext)
        except Exception,e:
            print e 
            return  ierror.WXBizMsgCrypt_EncryptAES_Error,None
    
    def decrypt(self,text,corpid):
        """对解密后的明文进行补位删除
        @param text: 密文 
        @return: 删除填充补位后的明文
        """
        try:
            cryptor = AES.new(self.key,self.mode,self.key[:16])
            # 使用BASE64对密文进行解码，然后AES-CBC解密
            plain_text  = cryptor.decrypt(base64.b64decode(text))
        except Exception,e:
            print e 
            return  ierror.WXBizMsgCrypt_DecryptAES_Error,None
        try:
            pad = ord(plain_text[-1]) 
            # 去掉补位字符串 
            #pkcs7 = PKCS7Encoder()
            #plain_text = pkcs7.encode(plain_text)   
            # 去除16位随机字符串
            content = plain_text[16:-pad]
            xml_len = socket.ntohl(struct.unpack("I",content[ : 4])[0])
            xml_content = content[4 : xml_len+4] 
            from_corpid = content[xml_len+4:]
        except Exception,e:
            print e
            return  ierror.WXBizMsgCrypt_IllegalBuffer,None
        if  from_corpid != corpid:
            return ierror.WXBizMsgCrypt_ValidateCorpid_Error,None
        return 0,xml_content
    
    def get_random_str(self):
        """ 随机生成16位字符串
        @return: 16位字符串
        """ 
        rule = string.letters + string.digits
        str = random.sample(rule, 16)
        return "".join(str)
        
class WXBizMsgCrypt(object):
    #构造函数
    #@param sToken: 公众平台上，开发者设置的Token
    # @param sEncodingAESKey: 公众平台上，开发者设置的EncodingAESKey
    # @param sCorpId: 企业号的CorpId
    def __init__(self,sToken,sEncodingAESKey,sCorpId):
        try:
            self.key = base64.b64decode(sEncodingAESKey+"=")  
            assert len(self.key) == 32
        except:
            throw_exception("[error]: EncodingAESKey unvalid !", FormatException) 
           #return ierror.WXBizMsgCrypt_IllegalAesKey)
        self.m_sToken = sToken
        self.m_sCorpid = sCorpId

		 #验证URL
         #@param sMsgSignature: 签名串，对应URL参数的msg_signature
         #@param sTimeStamp: 时间戳，对应URL参数的timestamp
         #@param sNonce: 随机串，对应URL参数的nonce
         #@param sEchoStr: 随机串，对应URL参数的echostr
         #@param sReplyEchoStr: 解密之后的echostr，当return返回0时有效
         #@return：成功0，失败返回对应的错误码	
    def VerifyURL(self, sMsgSignature, sTimeStamp, sNonce, sEchoStr):
        sha1 = SHA1()
        ret,signature = sha1.getSHA1(self.m_sToken, sTimeStamp, sNonce, sEchoStr)
        if ret  != 0:
            return ret, None
        if not signature == sMsgSignature:
            return ierror.WXBizMsgCrypt_ValidateSignature_Error, None
        # pc = Prpcrypt(self.key)
        # ret,sReplyEchoStr = pc.decrypt(sEchoStr,self.m_sCorpid)
        return ret,sEchoStr
	
    def EncryptMsg(self, sReplyMsg, sNonce, timestamp = None):
        #将公众号回复用户的消息加密打包
        #@param sReplyMsg: 企业号待回复用户的消息，xml格式的字符串
        #@param sTimeStamp: 时间戳，可以自己生成，也可以用URL参数的timestamp,如为None则自动用当前时间
        #@param sNonce: 随机串，可以自己生成，也可以用URL参数的nonce
        #sEncryptMsg: 加密后的可以直接回复用户的密文，包括msg_signature, timestamp, nonce, encrypt的xml格式的字符串,
        #return：成功0，sEncryptMsg,失败返回对应的错误码None     
        pc = Prpcrypt(self.key) 
        ret,encrypt = pc.encrypt(sReplyMsg, self.m_sCorpid)
        if ret != 0:
            return ret,None
        if timestamp is None:
            timestamp = str(int(time.time()))
        # 生成安全签名 
        sha1 = SHA1() 
        ret,signature = sha1.getSHA1(self.m_sToken, timestamp, sNonce, encrypt)
        if ret != 0: 
            return ret,None 
        xmlParse = XMLParse()  
        return ret,xmlParse.generate(encrypt, signature, timestamp, sNonce)  

    def DecryptMsg(self, sPostData, sMsgSignature, sTimeStamp, sNonce):
        # 检验消息的真实性，并且获取解密后的明文
        # @param sMsgSignature: 签名串，对应URL参数的msg_signature
        # @param sTimeStamp: 时间戳，对应URL参数的timestamp
        # @param sNonce: 随机串，对应URL参数的nonce
        # @param sPostData: 密文，对应POST请求的数据
        #  xml_content: 解密后的原文，当return返回0时有效
        # @return: 成功0，失败返回对应的错误码
         # 验证安全签名 
        xmlParse = XMLParse()
        ret,encrypt,touser_name = xmlParse.extract(sPostData)
        if ret != 0:
            return ret, None
        sha1 = SHA1() 
        ret,signature = sha1.getSHA1(self.m_sToken, sTimeStamp, sNonce, encrypt)
        if ret  != 0:
            return ret, None 
        if not signature == sMsgSignature:
            return ierror.WXBizMsgCrypt_ValidateSignature_Error, None
        pc = Prpcrypt(self.key)
        ret,xml_content = pc.decrypt(encrypt,self.m_sCorpid)
        return ret,xml_content 





'''
Created on 03 19, 2015

@author: tolerious

'''
from django.db import models
from models import *
import requests,json,logging,pprint

class Department(models.Model):
    name = models.CharField(max_length="100",default="")
    parentid = models.IntegerField(default=1)
    order = models.IntegerField(default=1)
    department_id = models.IntegerField(default=-1)

    def __unicode__(self):
        return self.name + ";" + str(self.department_id)

    def create_department(self):
        dic = {
            "name":self.name,
            "parentid":self.parentid,
        }
        accesstoken = AccessToken.objects.get(id=1).get_access_token()
        url = "https://qyapi.weixin.qq.com/cgi-bin/department/create?access_token=" + accesstoken
        data = json.dumps(dic)
        r = requests.post(url,data=data)
        return_dic = r.json()
        logging.info(return_dic)
        ids = return_dic["id"]
        self.department_id = ids
        return ids

    def update_department(self):
        dic = {
            "id": self.department_id,
            "name": self.name,
        }
        accesstoken = AccessToken.objects.get(id=1).get_access_token()
        url = "https://qyapi.weixin.qq.com/cgi-bin/department/update?access_token=" + accesstoken
        data = json.dumps(dic)
        r = requests.post(url,data=data)
        return_dic = r.json()
        logging.info(return_dic)
        return return_dic

    def delete_department(self):
        accesstoken = AccessToken.objects.get(id=1).get_access_token()
        url = "https://qyapi.weixin.qq.com/cgi-bin/department/delete?access_token=" + accesstoken + "&id=" + str(self.department_id)
        r = requests.get(url)
        return_dic = r.json()
        logging.info(return_dic)

    def get_department_list(self):
        accesstoken = AccessToken.objects.get(id=1).get_access_token()
        url = "https://qyapi.weixin.qq.com/cgi-bin/department/list?access_token=" + accesstoken + "&id=" + str(self.department_id)
        r = requests.get(url)
        return_dic = r.json()
        logging.info(return_dic)
        return return_dic

class Employee(models.Model):
    userid = models.CharField(default="",max_length=100,blank=False)
    name = models.CharField(default="",blank=False,max_length=100)
    department_list = models.TextField(default="[1]", blank=False)
    position = models.CharField(default="", max_length=100)
    mobile = models.CharField(default="",max_length=100)
    email = models.CharField(default="",max_length=100)
    weixinid = models.CharField(default="",max_length=100)
    extattr = models.TextField(default='{"attrs":[]}', blank=False)

    def __unicode__(self):
        return self.userid + ";" + self.weixinid

    def create_member(self):
        access_token = AccessToken.objects.get(id=1).get_access_token()
        url = "https://qyapi.weixin.qq.com/cgi-bin/user/create?access_token=" + access_token
        dump_department_list = json.loads(self.department_list)
        print self.extattr
        exta_dic = json.loads(self.extattr)
        dic = {
            "userid": self.userid,
            "name": self.name,
            "department":dump_department_list,
            "position":self.position,
            "mobile":self.mobile,
            "email":self.email,
            "weixinid":self.weixinid,
            "extattr":exta_dic,
        }
        data = json.dumps(dic)
        r = requests.post(url,data=data)
        return_json = r.json()
        logging.info(return_json)
        self.save()
        return return_json

    def update_member(self):
        access_token = AccessToken.objects.get(id=1).get_access_token()
        url = "https://qyapi.weixin.qq.com/cgi-bin/user/update?access_token=" + access_token
        dump_department_list = json.loads(self.department_list)
        print self.extattr
        exta_dic = json.loads(self.extattr)
        dic = {
            "userid": self.userid,
            "name": self.name,
            "department":dump_department_list,
            "position":self.position,
            "mobile":self.mobile,
            "email":self.email,
            "weixinid":self.weixinid,
            "extattr":exta_dic,
        }
        data = json.dumps(dic)
        r = requests.post(url,data=data)
        return_json = r.json()
        logging.info(return_json)
        return return_json

    def delete_member(self):
        access_token = AccessToken.objects.get(id=1).get_access_token()
        url = "https://qyapi.weixin.qq.com/cgi-bin/user/delete?access_token=" + access_token + "&userid=" + str(self.userid)
        r = requests.get(url)
        return_json = r.json()
        logging.info(return_json)
        return return_json

    def delete_bulk_members(self,delete_list):
        access_token = AccessToken.objects.get(id=1).get_access_token()
        url = "https://qyapi.weixin.qq.com/cgi-bin/user/batchdelete?access_token=" + access_token
        dic = {
            "useridlist": delete_list
        }
        data = json.dumps(dic)
        r = requests.post(url,data=data)
        return_json = r.json()
        logging.info(return_json)
        return return_json

    def get_employee_info(self):
        access_token = AccessToken.objects.get(id=1).get_access_token()
        url = "https://qyapi.weixin.qq.com/cgi-bin/user/get?access_token=" + access_token + "&userid=" + self.userid

        r = requests.get(url)
        return_json = r.json()
        logging.info(return_json)
        return return_json

    # may need to improve
    def get_department_employee_info(self,department_id,fetch_child="1",status="0"):
        access_token = AccessToken.objects.get(id=1).get_access_token()
        url = "https://qyapi.weixin.qq.com/cgi-bin/user/simplelist?access_token="+ access_token +"&department_id=" + str(department_id) + "&fetch_child="+ fetch_child+ "&status=" + status
        r = requests.get(url)
        return_json = r.json()
        logging.info(return_json)
        return return_json

    def get_department_employee_deatil_info(self,department_id,fetch_child="1",status="0"):
        pass


__author__ = 'tolerious'
from django.conf.urls import patterns, url

from django_weixin.views import basic, menu, api

urlpatterns = patterns('',
                       url(r'^validate/', basic.index, name='index'),
                       # url(r'^token/$', basic.accesstoken, name='accesstoken'),
                       url(r'^menu/config/', menu.create_menu_admin, name='menu_admin_page'),
                       url(r'^basic/menu/create/', menu.create_basic_menu, name='create_basic_menu'),
                       url(r'^access/token/get/', basic.get_access_token_view, name='get_access_token_view'),
                       url(r'^admin/dashboard/', basic.admin_dashboard, name='admin_dashboard'),
                       url(r'^show/ticket/', basic.get_qr_code_ticket, name='get_temp_qr_code'),

                       url(r'^api/create/menu/', api.create_meibo_menu, name='create_meibo_menu'),
                       url(r'^api/delete/menu', api.delete_menu, name="delete_meibo_menu"),
                       )
